; =====
; Timer Library
; =====
; Hobbyscripter <hobbyscripter@gmail.com>
; BLOG: http://hobbyscripter.blogspot.com/
; =====

; =====
; License
; =====
; This Library is a free software
; Copyright (c) 2006 Hobbyscripter

; This software is provided 'as-is', without any express or implied warranty.
; In no event will the authors be held liable for any damages arising from the
; use of this software.

; Permission is granted to anyone to use this software for any purpose,
; including commercial applications, and to alter it and redistribute
; it freely, subject to the following restrictions:
;     1. The origin of this software must not be misrepresented;
;        you must not claim that you wrote the original software. If
;        you use this software in a product, an acknowledgment in the
;        product documentation would be appreciated but is not required.
;     2. Altered source versions must be plainly marked as such, and
;        must not be misrepresented as being the original software.
;     3. This notice may not be removed or altered from any source
;        distribution.
; =====

; =====
; What's this?
; =====
; This is a function will calculate the second between
; 1970/01/01 00:00:00(GMT) ~ The input time
; =====

; =====
; Usage
; =====
; Push "yyyymmddhhmmss"
; Call Timer
; Pop $TheVARIAVLES
; =====

; =====
; Notice
; =====
; $(DefinitionTZHour) and $(DefinitionTZMinute) must be set.
; Usually it may be set by using LangString. You may find more
; information about LangString from NSIS's User Manual.
; =====


Var TIMERINPUT
Var TIMERIYEAR
Var TIMERIMONTH
Var TIMERIDATE
Var TIMERIHOUR
Var TIMERIMINUTE
Var TIMERISECOND
Var TIMERCALMONTH_FEB
Var TIMERCALYEARDIFF
Var TIMERCALTEMP1
Var TIMERCALTEMP2
Var TIMERCALTEMP3
Var TIMERCALTEMP4
Var TIMERCALTEMP5
Var TIMEROUTPUT

Function Timer
  Pop $TIMERINPUT
  StrCpy $TIMERIYEAR $TIMERINPUT 4 0
  StrCpy $TIMERIMONTH $TIMERINPUT 2 4
  StrCpy $TIMERIDATE $TIMERINPUT 2 6
  StrCpy $TIMERIHOUR $TIMERINPUT 2 8
  StrCpy $TIMERIMINUTE $TIMERINPUT 2 10
  StrCpy $TIMERISECOND $TIMERINPUT 2 12

  ; GMTFix:
  ; Change the time into Greenwich Mean Time (GMT)
  IntOp $TIMERIHOUR $TIMERIHOUR - $(DefinitionTZHour)
  IntOp $TIMERIMINUTE $TIMERIMINUTE - $(DefinitionTZMinute)

  ;CheckLeapYear:
  IntOp $TIMERCALTEMP1 $TIMERIYEAR % 4 ; year mod 4
  IntCmp $TIMERCALTEMP1 0 0 IsntLeapYear IsntLeapYear
    IntOp $TIMERCALTEMP1 $TIMERIYEAR % 100 ; year mod 100
    IntCmp $TIMERCALTEMP1 0 0 IsLeapYear IsLeapYear
      IntOp $TIMERCALTEMP1 $TIMERIYEAR % 400 ; year mod 400
      IntCmp $TIMERCALTEMP1 0 IsLeapYear IsntLeapYear IsntLeapYear
  IsLeapYear:
    IntOp $TIMERCALMONTH_FEB 29 + 0
    Goto EndOfLeapYear
  IsntLeapYear:
    IntOp $TIMERCALMONTH_FEB 28 + 0
  EndOfLeapYear:

  ;CalThisMonthSecond:
  ; Note: this part is calculating the second from:
  ;       $TIMERIYEAR/$TIMERIMONTH/01 00:00:00 ~
  ;       $TIMERIYEAR/$TIMERIMONTH/$TIMERIDATE $TIMERIHOUR:$TIMERIMINUTE:$TIMERISECOND
  IntOp $TIMEROUTPUT 0 + $TIMERISECOND ; This second
  IntOp $TIMERCALTEMP1 $TIMERIMINUTE * 60
  IntOp $TIMEROUTPUT $TIMEROUTPUT + $TIMERCALTEMP1 ; $TIMERCALTEMP1 = This minute * 60 (second)
  IntOp $TIMERCALTEMP1 $TIMERIHOUR * 3600
  IntOp $TIMEROUTPUT $TIMEROUTPUT + $TIMERCALTEMP1 ; $TIMERCALTEMP1 = This hour * 3600 (second)
  IntOp $TIMERCALTEMP1 $TIMERIDATE - 1
  IntOp $TIMERCALTEMP1 $TIMERCALTEMP1 * 86400
  IntOp $TIMEROUTPUT $TIMEROUTPUT + $TIMERCALTEMP1 ; $TIMERCALTEMP1 = This date * 86400 (second)

  ;CalThisYearBeforeMonthSecond:
  ; Note: this part is calculating the second from:
  ;       $TIMERIYEAR/01/01 00:00:00 ~
  ;       $TIMERIYEAR/$TIMERIMONTH-1/$DayCount(TIMERIMONTH-1) 11:59:59
  ; Note: now $TIMERCALTEMP1 will be the count of the day
  IntOp $TIMERCALTEMP1 0 + 0 ; Initialization
  IntCmp $TIMERIMONTH 1 +2 +2 0 ; if including JANUARY
    IntOp $TIMERCALTEMP1 $TIMERCALTEMP1 + 31
  IntCmp $TIMERIMONTH 2 +2 +2 0 ; if including FEBRUARY
    IntOp $TIMERCALTEMP1 $TIMERCALMONTH_FEB + 31
  IntCmp $TIMERIMONTH 3 +2 +2 0 ; if including MARCH
    IntOp $TIMERCALTEMP1 $TIMERCALTEMP1 + 31
  IntCmp $TIMERIMONTH 4 +2 +2 0 ; if including APRIL
    IntOp $TIMERCALTEMP1 $TIMERCALTEMP1 + 30
  IntCmp $TIMERIMONTH 5 +2 +2 0 ; if including MAY
    IntOp $TIMERCALTEMP1 $TIMERCALTEMP1 + 31
  IntCmp $TIMERIMONTH 6 +2 +2 0 ; if including JUNE
    IntOp $TIMERCALTEMP1 $TIMERCALTEMP1 + 30
  IntCmp $TIMERIMONTH 7 +2 +2 0 ; if including JULY
    IntOp $TIMERCALTEMP1 $TIMERCALTEMP1 + 31
  IntCmp $TIMERIMONTH 8 +2 +2 0 ; if including AUGUST
    IntOp $TIMERCALTEMP1 $TIMERCALTEMP1 + 31
  IntCmp $TIMERIMONTH 9 +2 +2 0 ; if including SETEMBER
    IntOp $TIMERCALTEMP1 $TIMERCALTEMP1 + 30
  IntCmp $TIMERIMONTH 10 +2 +2 0 ; if including OCTOBER
    IntOp $TIMERCALTEMP1 $TIMERCALTEMP1 + 31
  IntCmp $TIMERIMONTH 11 +2 +2 0 ; if including NOVEMBER
    IntOp $TIMERCALTEMP1 $TIMERCALTEMP1 + 30
  IntOp $TIMERCALTEMP1 $TIMERCALTEMP1 * 86400 ; Convert $TIMERCALTEMP1 into second
  IntOp $TIMEROUTPUT $TIMEROUTPUT + $TIMERCALTEMP1

  ;CalAllBeforeYearSecond:
  IntOp $TIMERCALYEARDIFF $TIMERIYEAR - 1970
  IntOp $TIMERCALTEMP1 $TIMERCALYEARDIFF % 4 ; year mod 4
  IntOp $TIMERCALTEMP1 $TIMERCALYEARDIFF - $TIMERCALTEMP1
  IntOp $TIMERCALTEMP1 $TIMERCALTEMP1 / 4 ; $TIMERCALTEMP1 = Leap Year Count only in mod 4
  IntOp $TIMERCALTEMP2 $TIMERCALYEARDIFF % 100 ; year mod 100
  IntOp $TIMERCALTEMP2 $TIMERCALYEARDIFF - $TIMERCALTEMP2
  IntOp $TIMERCALTEMP2 $TIMERCALTEMP2 / 100 ; $TIMERCALTEMP2 = Leap Year Count only in mod 100
  IntOp $TIMERCALTEMP3 $TIMERCALYEARDIFF % 400 ; year mod 400
  IntOp $TIMERCALTEMP3 $TIMERCALYEARDIFF - $TIMERCALTEMP3
  IntOp $TIMERCALTEMP3 $TIMERCALTEMP3 / 400 ; $TIMERCALTEMP3 = Leap Year Count only in mod 400
  IntOp $TIMERCALTEMP4 $TIMERCALTEMP1 - $TIMERCALTEMP2 ; Count 4 - 100
  IntOp $TIMERCALTEMP4 $TIMERCALTEMP4 + $TIMERCALTEMP3 ; Count 4 - 100 + 400, now $TIMERCALTEMP4 is all the leap years
  IntOp $TIMERCALTEMP5 $TIMERCALYEARDIFF - $TIMERCALTEMP4 ; $TIMERCALTEMP5 = all years which is not leap year
  IntOp $TIMERCALTEMP4 $TIMERCALTEMP4 * 31622400 ; Convert $TIMERCALTEMP4 into second
  IntOp $TIMERCALTEMP5 $TIMERCALTEMP5 * 31536000 ; Convert $TIMERCALTEMP5 into second
  IntOp $TIMEROUTPUT $TIMEROUTPUT + $TIMERCALTEMP4
  IntOp $TIMEROUTPUT $TIMEROUTPUT + $TIMERCALTEMP5

  Push $TIMEROUTPUT
FunctionEnd